@@ -1,3 +1,4 @@
+/* vim: set noet ts=8 sw=8 sts=0: */
 /* Tracker Extract - extracts embedded metadata from files
  * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
  *
@@ -17,7 +18,6 @@
  * Boston, MA  02110-1301, USA.
  */
 
-
 #include "config.h"
 
 #ifdef HAVE_POPPLER
@@ -26,40 +26,57 @@
 #include <string.h>
 #include <glib.h>
 
-void tracker_extract_pdf (gchar *filename, GHashTable *metadata)
+#include "tracker-extract.h"
+
+
+void
+tracker_extract_pdf (gchar *filename, GHashTable *metadata)
 {
 	PopplerDocument *document;
 	gchar           *tmp;
-	gchar           *title;
-	gchar           *author;
-	gchar           *subject;
-	gchar           *keywords;
+	gchar           *title          = NULL;
+	gchar           *author         = NULL;
+	gchar           *subject        = NULL;
+	gchar           *keywords       = NULL;
+	gchar           *metadata_xml   = NULL;
 	GTime            creation_date;
-	GError          *error = NULL;
+	GError          *error          = NULL;
 
 	g_type_init ();
+
 	tmp = g_strconcat ("file://", filename, NULL);
 	document = poppler_document_new_from_file (tmp, NULL, &error);
 	g_free (tmp);
-	if (document == NULL || error)
+
+	if (document == NULL || error) {
 		return;
+        }
 
 	g_object_get (document,
-		"title", &title,
-		"author", &author,
-		"subject", &subject,
-		"keywords", &keywords,
-		"creation-date", &creation_date,
-		NULL);
+                      "title", &title,
+                      "author", &author,
+                      "subject", &subject,
+                      "keywords", &keywords,
+                      "creation-date", &creation_date,
+                      NULL);
 
-	if (title && strlen (title))
+	/* metadata property not present in older poppler versions */
+	if (g_object_class_find_property (G_OBJECT_GET_CLASS (document), "metadata")) {
+		g_object_get (document, "metadata", &metadata_xml, NULL);
+        }
+
+	if (!tracker_is_empty_string (title)) {
 		g_hash_table_insert (metadata, g_strdup ("Doc:Title"), g_strdup (title));
-	if (author && strlen (author))
+        }
+	if (!tracker_is_empty_string (author)) {
 		g_hash_table_insert (metadata, g_strdup ("Doc:Author"), g_strdup (author));
-	if (subject && strlen (subject))
+        }
+	if (!tracker_is_empty_string (subject)) {
 		g_hash_table_insert (metadata, g_strdup ("Doc:Subject"), g_strdup (subject));
-	if (keywords && strlen (keywords))
+        }
+	if (!tracker_is_empty_string (keywords)) {
 		g_hash_table_insert (metadata, g_strdup ("Doc:Keywords"), g_strdup (keywords));
+        }
 
 
 #if 0
@@ -71,15 +88,19 @@ void tracker_extract_pdf (gchar *filename, GHashTable *metadata)
 	g_hash_table_insert (metadata, g_strdup ("Doc:PageCount"),
 		g_strdup_printf ("%d", poppler_document_get_n_pages (document)));
 
+	if ( metadata_xml ) {
+		tracker_read_xmp (metadata_xml, strlen (metadata_xml), metadata);
+	}
+
 	g_free (title);
 	g_free (author);
 	g_free (subject);
 	g_free (keywords);
+	g_free (metadata_xml);
+
 	g_object_unref (document);
 }
 
 #else
 #warning "Not building PDF metadata extractor."
 #endif  /* HAVE_POPPLER */
-
-
