@@ -26,8 +26,6 @@
  * Boston, MA  02110-1301, USA.
  */
 
-
-
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -54,16 +52,16 @@
 #define C_STANDARD_NUMERIC_STRFTIME_CHARACTERS "dHIjmMSUwWyY"
 #define SUS_EXTENDED_STRFTIME_MODIFIERS "EO"
 #define BINARY_EXEC_MIME_TYPE      "application/x-executable"
-#define ICON_THEME_EXECUTABLE_ICON "gnome-fs-executable"
-#define ICON_THEME_REGULAR_ICON    "gnome-fs-regular"
-#define ICON_THEME_CHAR_DEVICE     "gnome-fs-chardev"
-#define ICON_THEME_BLOCK_DEVICE    "gnome-fs-blockdev"
-#define ICON_THEME_SOCKET          "gnome-fs-socket"
-#define ICON_THEME_FIFO            "gnome-fs-fifo"
 #define MAX_SYMLINKS_FOLLOWED      32
 #define GSEARCH_DATE_FORMAT_LOCALE "locale"
 #define GSEARCH_DATE_FORMAT_ISO    "iso"
 
+gboolean
+tracker_is_empty_string (const gchar *s)
+{
+	return s == NULL || s[0] == '\0';
+}
+
 /* START OF THE GCONF FUNCTIONS */
 
 static gboolean
@@ -87,7 +85,7 @@ tracker_search_gconf_client_get_global (void)
 
 	/* Initialize gconf if needed */
 	if (!gconf_is_initialized ()) {
-		char *argv[] = { "tracker_search-preferences", NULL };
+		gchar *argv[] = { "tracker_search-preferences", NULL };
 		GError *error = NULL;
 
 		if (!gconf_init (1, argv, &error)) {
@@ -104,7 +102,7 @@ tracker_search_gconf_client_get_global (void)
 	return global_gconf_client;
 }
 
-char *
+gchar *
 tracker_search_gconf_get_string (const gchar * key)
 {
 	GConfClient * client;
@@ -127,7 +125,7 @@ tracker_search_gconf_get_string (const gchar * key)
 
 GSList *
 tracker_search_gconf_get_list (const gchar * key,
-                            GConfValueType list_type)
+			       GConfValueType list_type)
 {
 	GConfClient * client;
 	GError * error = NULL;
@@ -149,8 +147,8 @@ tracker_search_gconf_get_list (const gchar * key,
 
 void
 tracker_search_gconf_set_list (const gchar * key,
-                            GSList * list,
-                            GConfValueType list_type)
+			       GSList * list,
+			       GConfValueType list_type)
 {
 	GConfClient * client;
 	GError * error = NULL;
@@ -188,7 +186,7 @@ tracker_search_gconf_get_int (const gchar * key)
 
 void
 tracker_search_gconf_set_int (const gchar * key,
-                           const gint value)
+			      const gint value)
 {
 	GConfClient * client;
 	GError * error = NULL;
@@ -226,7 +224,7 @@ tracker_search_gconf_get_boolean (const gchar * key)
 
 void
 tracker_search_gconf_set_boolean (const gchar * key,
-                               const gboolean flag)
+				  const gboolean flag)
 {
 	GConfClient * client;
 	GError * error = NULL;
@@ -262,9 +260,9 @@ tracker_search_gconf_add_dir (const gchar * dir)
 
 void
 tracker_search_gconf_watch_key (const gchar * dir,
-                             const gchar * key,
-                             GConfClientNotifyFunc callback,
-                             gpointer user_data)
+				const gchar * key,
+				GConfClientNotifyFunc callback,
+				gpointer user_data)
 {
 	GConfClient * client;
 	GError * error = NULL;
@@ -356,7 +354,7 @@ is_quick_search_excluded_path (const gchar * path)
 		gchar * dir;
 
 		/* Skip empty or null values. */
-		if ((tmp_list->data == NULL) || (strlen (tmp_list->data) == 0)) {
+		if (tracker_is_empty_string (tmp_list->data)) {
 			continue;
 		}
 
@@ -434,7 +432,7 @@ is_second_scan_excluded_path (const gchar * path)
 		gchar * dir;
 
 		/* Skip empty or null values. */
-		if ((tmp_list->data == NULL) || (strlen (tmp_list->data) == 0)) {
+		if (tracker_is_empty_string (tmp_list->data)) {
 			continue;
 		}
 
@@ -516,6 +514,39 @@ limit_string_to_x_lines (GString * string,
 	return FALSE;
 }
 
+gchar *
+tracker_string_replace (const gchar * haystack,
+			gchar * needle,
+			gchar * replacement)
+{
+        GString * str;
+        gint pos, needle_len;
+
+	g_return_val_if_fail (haystack && needle, NULL);
+
+	needle_len = strlen (needle);
+
+        str = g_string_new ("");
+
+        for (pos = 0; haystack[pos]; pos++)
+        {
+                if (strncmp (&haystack[pos], needle, needle_len) == 0)
+                {
+
+			if (replacement) {
+	                        str = g_string_append (str, replacement);
+			}
+
+                        pos += needle_len - 1;
+
+                } else {
+                        str = g_string_append_c (str, haystack[pos]);
+		}
+        }
+
+        return g_string_free (str, FALSE);
+}
+
 static gint
 count_of_char_in_string (const gchar * string,
 			 const gchar c)
@@ -657,7 +688,7 @@ get_readable_date (const gchar * format_string,
 
 gchar *
 tracker_search_strdup_strftime (const gchar * format,
-                             struct tm * time_pieces)
+				struct tm * time_pieces)
 {
 	/* This function is borrowed from eel's eel_strdup_strftime() */
 	GString * string;
@@ -1067,10 +1098,10 @@ tracker_search_get_thumbnail_image (const gchar * file)
 
 static gchar *
 tracker_search_icon_lookup (GSearchWindow * gsearch,
-                         const gchar * file,
-                         const gchar * mime,
-                         GnomeVFSFileInfo * file_info,
-                         gboolean enable_thumbnails)
+			    const gchar * file,
+			    const gchar * mime,
+			    GnomeVFSFileInfo * file_info,
+			    gboolean enable_thumbnails)
 {
 	GnomeIconLookupFlags lookup_flags = GNOME_ICON_LOOKUP_FLAGS_NONE;
 	gchar * icon_name = NULL;
@@ -1078,16 +1109,14 @@ tracker_search_icon_lookup (GSearchWindow * gsearch,
 
 	uri = gnome_vfs_get_uri_from_local_path (file);
 
-//	if ((enable_thumbnails == TRUE) && (gsearch->show_thumbnails == TRUE)) {
-		if ((strncmp (mime, "image/", 6) != 0) ||
-	    	    ((int)file_info->size < (int)gsearch->show_thumbnails_file_size_limit)) {
-		    	if (gsearch->thumbnail_factory == NULL) {
-			    	gsearch->thumbnail_factory = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
-			}
-			lookup_flags = GNOME_ICON_LOOKUP_FLAGS_SHOW_SMALL_IMAGES_AS_THEMSELVES |
-			               GNOME_ICON_LOOKUP_FLAGS_ALLOW_SVG_AS_THEMSELVES;
+	if ((strncmp (mime, "image/", 6) != 0) ||
+    	    ((int)file_info->size < (int)gsearch->show_thumbnails_file_size_limit)) {
+	    	if (gsearch->thumbnail_factory == NULL) {
+		    	gsearch->thumbnail_factory = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
 		}
-//	}
+		lookup_flags = GNOME_ICON_LOOKUP_FLAGS_SHOW_SMALL_IMAGES_AS_THEMSELVES |
+		               GNOME_ICON_LOOKUP_FLAGS_ALLOW_SVG_AS_THEMSELVES;
+	}
 
 	icon_name = gnome_icon_lookup (gtk_icon_theme_get_default (),
 	                               gsearch->thumbnail_factory,
@@ -1107,12 +1136,13 @@ get_file_pixbuf (GSearchWindow * gsearch,
 		 const char *mime,
                  GnomeVFSFileInfo * file_info)
 {
-	GdkPixbuf * pixbuf;
+	GdkPixbuf * pixbuf = NULL;
 	gchar * icon_name = NULL;
 
 	if (file == NULL || mime == NULL) {
 		icon_name = g_strdup (ICON_THEME_REGULAR_ICON);
-	}
+	} 
+
 	else if ((g_file_test (file, G_FILE_TEST_IS_EXECUTABLE)) &&
 	         (g_ascii_strcasecmp (mime, "application/x-executable-binary") == 0)) {
 		icon_name = g_strdup (ICON_THEME_EXECUTABLE_ICON);
@@ -1128,91 +1158,73 @@ get_file_pixbuf (GSearchWindow * gsearch,
 	}
 	else if (g_ascii_strcasecmp (mime, "x-special/fifo") == 0) {
 		icon_name = g_strdup (ICON_THEME_FIFO);
-	}
-	else {
-		icon_name = tracker_search_icon_lookup (gsearch, file, mime, file_info, TRUE);
-	}
-
-	pixbuf = (GdkPixbuf *) g_hash_table_lookup (gsearch->search_results_filename_hash_table, icon_name);
-
-	if (pixbuf == NULL) {
+	} else {
 
+		/* check for thumbnail first */
+		GdkPixbuf *thumbnail_pixbuf = tracker_search_get_thumbnail_image (file);
 		
+		if (thumbnail_pixbuf != NULL) {
 
-			if ((strncmp (mime, "image/", 6) != 0) ||
-	    		    ((int)file_info->size < (int)gsearch->show_thumbnails_file_size_limit)) {
+			if ((gdk_pixbuf_get_width (thumbnail_pixbuf) > ICON_SIZE) ||
+			    (gdk_pixbuf_get_height (thumbnail_pixbuf) > ICON_SIZE)) {
 
-				if (strcmp (icon_name, file) == 0) {
-					pixbuf = gdk_pixbuf_new_from_file_at_scale (file, ICON_SIZE, ICON_SIZE, TRUE, NULL);
+				gfloat scale_factor_x = 1.0;
+				gfloat scale_factor_y = 1.0;
+				gint scale_x;
+				gint scale_y;
 
-					if (pixbuf == NULL) {
-						icon_name = tracker_search_icon_lookup (gsearch, file, mime, file_info, FALSE);
-					}
+				if (gdk_pixbuf_get_width (thumbnail_pixbuf) > ICON_SIZE) {
+					scale_factor_x = (gfloat) ICON_SIZE / (gfloat) gdk_pixbuf_get_width (thumbnail_pixbuf);
+				}
+				if (gdk_pixbuf_get_height (thumbnail_pixbuf) > ICON_SIZE) {
+					scale_factor_y = (gfloat) ICON_SIZE / (gfloat) gdk_pixbuf_get_height (thumbnail_pixbuf);
 				}
 
-				if (pixbuf == NULL) {
-					pixbuf = tracker_search_get_thumbnail_image (file);
-				}
-			}
-		
-
-		if (pixbuf == NULL) {
-
-			GdkPixbuf * thumbnail_pixbuf = NULL;
-
-			thumbnail_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon_name,
-			                                             ICON_SIZE, 0, NULL);
-
-			if (thumbnail_pixbuf != NULL) {
-
-				/*  The following is a workaround for bugzilla report #311318,                   */
-				/*  "gtk_icon_theme_load_icon () can return pixbufs larger than requested size". */
-				/*  Please see the url, http://bugzilla.gnome.org/show_bug.cgi?id=311318.        */
-
-				if ((gdk_pixbuf_get_width (thumbnail_pixbuf) > ICON_SIZE) ||
-				    (gdk_pixbuf_get_height (thumbnail_pixbuf) > ICON_SIZE)) {
-
-					gfloat scale_factor_x = 1.0;
-					gfloat scale_factor_y = 1.0;
-					gint scale_x;
-					gint scale_y;
-
-					if (gdk_pixbuf_get_width (thumbnail_pixbuf) > ICON_SIZE) {
-						scale_factor_x = (gfloat) ICON_SIZE / (gfloat) gdk_pixbuf_get_width (thumbnail_pixbuf);
-					}
-					if (gdk_pixbuf_get_height (thumbnail_pixbuf) > ICON_SIZE) {
-						scale_factor_y = (gfloat) ICON_SIZE / (gfloat) gdk_pixbuf_get_height (thumbnail_pixbuf);
-					}
-
-					if (gdk_pixbuf_get_width (thumbnail_pixbuf) > gdk_pixbuf_get_height (thumbnail_pixbuf)) {
-						scale_x = ICON_SIZE;
-						scale_y = (gint) (gdk_pixbuf_get_height (thumbnail_pixbuf) * scale_factor_x);
-					}
-					else {
-						scale_x = (gint) (gdk_pixbuf_get_width (thumbnail_pixbuf) * scale_factor_y);
-						scale_y = ICON_SIZE;
-					}
-
-					pixbuf = gdk_pixbuf_scale_simple (thumbnail_pixbuf, scale_x, scale_y, GDK_INTERP_BILINEAR);
-					g_object_unref (thumbnail_pixbuf);
+				if (gdk_pixbuf_get_width (thumbnail_pixbuf) > gdk_pixbuf_get_height (thumbnail_pixbuf)) {
+					scale_x = ICON_SIZE;
+					scale_y = (gint) (gdk_pixbuf_get_height (thumbnail_pixbuf) * scale_factor_x);
 				}
 				else {
-					pixbuf = thumbnail_pixbuf;
+					scale_x = (gint) (gdk_pixbuf_get_width (thumbnail_pixbuf) * scale_factor_y);
+					scale_y = ICON_SIZE;
 				}
+
+				pixbuf = gdk_pixbuf_scale_simple (thumbnail_pixbuf, scale_x, scale_y, GDK_INTERP_BILINEAR);
+				g_object_unref (thumbnail_pixbuf);
+
+				return pixbuf;
+
+			} else {
+				return thumbnail_pixbuf;
 			}
+		
 		}
 
-		if (pixbuf != NULL) {
-			g_hash_table_insert (gsearch->search_results_filename_hash_table, icon_name, pixbuf);
+		/* check if image can be generated from file */
+
+		if ((strncmp (mime, "image/", 6) == 0)) {
+			pixbuf = gdk_pixbuf_new_from_file_at_scale (file, ICON_SIZE, ICON_SIZE, TRUE, NULL);
 		}
-		else {
-			g_free (icon_name);
+
+		if (pixbuf) {
+			return pixbuf;
+		} else {
+			icon_name = tracker_search_icon_lookup (gsearch, file, mime, file_info, TRUE);
 		}
+
+		
 	}
-	else {
-		g_free (icon_name);
+
+	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon_name, ICON_SIZE, 0, NULL);
+
+	if (!pixbuf) {
+		gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), ICON_THEME_REGULAR_ICON, ICON_SIZE, 0, NULL); 
 	}
 
+		
+	g_free (icon_name);
+
+
 	return pixbuf;
 }
 
@@ -1221,9 +1233,13 @@ open_file_with_xdg_open (GtkWidget * window,
                          const gchar * file)
 {
 	gboolean  result;
-	gchar    *command = g_strconcat ("xdg-open '", file, "'", NULL);
+	gchar     *quoted_filename = g_shell_quote (file);
+	gchar     *command = g_strconcat ("xdg-open ", quoted_filename, NULL);
+
+	g_free (quoted_filename);
 	result = g_spawn_command_line_async (command, NULL);
 	g_free (command);
+
 	return result;
 }
 
@@ -1499,9 +1515,29 @@ tracker_search_set_columns_order (GtkTreeView * treeview)
 	g_slist_free (order);
 }
 
+gint
+tracker_get_stored_separator_position (void)
+{
+	gint saved_pos;
+
+	saved_pos = tracker_search_gconf_get_int ("/apps/tracker-search-tool/separator_position");
+
+	if (saved_pos < 1) {
+		return DEFAULT_SEPARATOR_POSITION;
+	}
+
+	return saved_pos;	
+}
+
+void
+tracker_set_stored_separator_position (gint pos)
+{
+	tracker_search_gconf_set_int ("/apps/tracker-search-tool/separator_position", pos);
+}
+
 void
 tracker_search_get_stored_window_geometry (gint * width,
-                                        gint * height)
+					   gint * height)
 {
 	gint saved_width;
 	gint saved_height;
@@ -1513,11 +1549,11 @@ tracker_search_get_stored_window_geometry (gint * width,
 	saved_width = tracker_search_gconf_get_int ("/apps/tracker-search-tool/default_window_width");
 	saved_height = tracker_search_gconf_get_int ("/apps/tracker-search-tool/default_window_height");
 
-	if (saved_width == -1) {
+	if (saved_width < 1) {
 		saved_width = DEFAULT_WINDOW_WIDTH;
 	}
 
-	if (saved_height == -1) {
+	if (saved_height < 1) {
 		saved_height = DEFAULT_WINDOW_HEIGHT;
 	}
 
