@@ -21,6 +21,157 @@
 
 #include "tracker-dbus-methods.h"
 #include "tracker-rdf-query.h"
+#include "tracker-indexer.h"
+
+extern Tracker *tracker;
+
+
+void
+tracker_dbus_method_search_get_hit_count (DBusRec *rec)
+{
+	DBConnection *db_con;
+	DBusError    dbus_error;
+	gchar	     *service;
+	gchar	     *str;
+
+	g_return_if_fail (rec);
+	g_return_if_fail (rec->user_data);
+
+	db_con = rec->user_data;
+
+/*
+	<!--  returns no of hits for the search_text on the servce -->
+		<method name="GetHitCount">
+			<arg type="s" name="service" direction="in" />
+			<arg type="s" name="search_text" direction="in" />
+			<arg type="i" name="result" direction="out" />
+		</method>
+
+	
+*/
+
+	dbus_error_init (&dbus_error);
+	if (!dbus_message_get_args (rec->message, NULL, 
+			       DBUS_TYPE_STRING, &service,
+			       DBUS_TYPE_STRING, &str,
+			       DBUS_TYPE_INVALID)) {
+		tracker_set_error (rec, "DBusError: %s;%s", dbus_error.name, dbus_error.message);
+		dbus_error_free (&dbus_error);
+		return;
+	}
+
+	if (!service) {
+		tracker_set_error (rec, "No service was specified");
+		return;
+	}
+
+	if (!tracker_is_valid_service (db_con, service)) {
+		tracker_set_error (rec, "Invalid service %s or service has not been implemented yet", service);
+		return;
+	}
+
+	if (tracker_is_empty_string (str)) {
+		tracker_set_error (rec, "No search term was specified");
+		return;
+	}
+
+	//tracker_log ("Executing GetHitCount with params %s, %s", service, str);
+
+	gchar **array;
+	gint service_array[1];
+	gint result;
+	DBusMessage *reply;
+
+	service_array[0] = tracker_get_id_for_service (service);
+
+	db_con = tracker_db_get_service_connection (db_con, service);
+
+	SearchQuery *query = tracker_create_query (db_con->word_index, service_array, 1, 0, 999999);
+
+	array = tracker_parse_text_into_array (str);
+
+	gchar **pstr;
+
+	for (pstr = array; *pstr; pstr++) {
+		tracker_add_query_word (query, *pstr, WordNormal);
+	}
+
+	g_strfreev (array);
+
+	result = tracker_get_hit_count (query);
+
+	tracker_free_query (query);
+
+	reply = dbus_message_new_method_return (rec->message);
+
+	dbus_message_append_args (reply,
+	  			  DBUS_TYPE_INT32,
+				  &result,
+	  			  DBUS_TYPE_INVALID);
+
+	dbus_connection_send (rec->connection, reply, NULL);
+
+	dbus_message_unref (reply);
+}
+
+
+void
+tracker_dbus_method_search_get_hit_count_all (DBusRec *rec)
+{
+	DBConnection *db_con;
+	DBusError    dbus_error;
+	gchar	     *str;
+
+	g_return_if_fail (rec);
+	g_return_if_fail (rec->user_data);
+
+	db_con = rec->user_data;
+
+/*
+		<!--  returns [service name, no. of hits] for the search_text -->
+		<method name="GetHitCountAll">
+			<arg type="s" name="search_text" direction="in" />
+			<arg type="aas" name="result" direction="out" />
+		</method>
+	
+*/
+
+	dbus_error_init (&dbus_error);
+	if (!dbus_message_get_args (rec->message, NULL, 
+			       DBUS_TYPE_STRING, &str,
+			       DBUS_TYPE_INVALID)) {
+		tracker_set_error (rec, "DBusError: %s;%s", dbus_error.name, dbus_error.message);
+		dbus_error_free (&dbus_error);
+		return;
+	}
+
+	if (tracker_is_empty_string (str)) {
+		tracker_set_error (rec, "No search term was specified");
+		return;
+	}
+
+	//tracker_log ("Executing detailed search with params %s, %s, %d, %d", service, str, offset, limit);
+
+	gchar ***res, **array;
+
+	SearchQuery *query = tracker_create_query (db_con->word_index, NULL, 0, 0, 999999);
+
+	array = tracker_parse_text_into_array (str);
+
+	gchar **pstr;
+
+	for (pstr = array; *pstr; pstr++) {
+		tracker_add_query_word (query, *pstr, WordNormal);	
+	}
+
+	g_strfreev (array);
+
+	res = tracker_get_hit_counts (query);
+
+	tracker_free_query (query);	
+
+	tracker_dbus_reply_with_query_result (rec, res);
+}
 
 
 void
@@ -29,14 +180,15 @@ tracker_dbus_method_search_text (DBusRec *rec)
 	DBConnection *db_con;
 	DBusError    dbus_error;
 	DBusMessage  *reply;
-	char	     **array;
-	int	     row_count, i;
-	int	     limit, query_id, offset;
-	char	     *service;
-	char	     ***res;
-	char	     *str;
+	gchar	     **array;
+	gint	     row_count, i;
+	gint	     limit, query_id, offset;
+	gchar	     *service;
+	gchar	     ***res;
+	gchar	     *str;
 
-	g_return_if_fail (rec && rec->user_data);
+	g_return_if_fail (rec);
+	g_return_if_fail (rec->user_data);
 
 	db_con = rec->user_data;
 
@@ -65,7 +217,7 @@ tracker_dbus_method_search_text (DBusRec *rec)
 		return;
 	}
 
-	if (!service)  {
+	if (!service) {
 		tracker_set_error (rec, "No service was specified");
 		return;
 	}
@@ -75,17 +227,19 @@ tracker_dbus_method_search_text (DBusRec *rec)
 		return;
 	}
 
-	if (!str || strlen (str) == 0) {
+	if (tracker_is_empty_string (str)) {
 		tracker_set_error (rec, "No search term was specified");
 		return;
 	}
 
-	if (limit < 0) {
+	if (limit < 1) {
 		limit = 1024;
 	}
 
 	tracker_log ("Executing search with params %s, %s", service, str);
 
+	db_con = tracker_db_get_service_connection (db_con, service);
+
 	res = tracker_db_search_text (db_con, service, str, offset, limit, FALSE, FALSE);
 
 	row_count = 0;
@@ -96,9 +250,9 @@ tracker_dbus_method_search_text (DBusRec *rec)
 		row_count = tracker_get_row_count (res);
 
 		if (row_count > 0) {
-			char **row;
+			gchar **row;
 
-			array = g_new (char *, row_count);
+			array = g_new (gchar *, row_count);
 
 			i = 0;
 
@@ -117,7 +271,7 @@ tracker_dbus_method_search_text (DBusRec *rec)
 		tracker_db_free_result (res);
 
 	} else {
-		array = g_new (char *, 1);
+		array = g_new (gchar *, 1);
 		array[0] = NULL;
 	}
 
@@ -140,22 +294,21 @@ tracker_dbus_method_search_text (DBusRec *rec)
 }
 
 
-
 void
 tracker_dbus_method_search_text_detailed (DBusRec *rec)
 {
 	DBConnection *db_con;
 	DBusError    dbus_error;
-	int	     limit, query_id, offset;
-	char	     *service;
-	char	     ***res;
-	char	     *str;
+	gint	     limit, query_id, offset;
+	gchar	     *service;
+	gchar	     ***res;
+	gchar	     *str;
 
-	g_return_if_fail (rec && rec->user_data);
+	g_return_if_fail (rec);
+	g_return_if_fail (rec->user_data);
 
 	db_con = rec->user_data;
 
-
 /*
 		More detailed version of above. Searches specified service for entities that match the specified search_text. 
 		Returns hits in array format [uri, service, mime]  -->
@@ -181,7 +334,7 @@ tracker_dbus_method_search_text_detailed (DBusRec *rec)
 		return;
 	}
 
-	if (!service)  {
+	if (!service) {
 		tracker_set_error (rec, "No service was specified");
 		return;
 	}
@@ -191,22 +344,28 @@ tracker_dbus_method_search_text_detailed (DBusRec *rec)
 		return;
 	}
 
-	if (!str || strlen (str) == 0) {
+	if (tracker_is_empty_string (str)) {
 		tracker_set_error (rec, "No search term was specified");
 		return;
 	}
 
-	if (limit < 0) {
+	if (limit < 1) {
 		limit = 1024;
 	}
 
 	tracker_log ("Executing detailed search with params %s, %s, %d, %d", service, str, offset, limit);
 
+	db_con = tracker_db_get_service_connection (db_con, service);
+
 	res = tracker_db_search_text (db_con, service, str, offset, limit, FALSE, TRUE);
 
+	if (tracker->verbosity > 0) {
+		tracker_db_log_result (res);
+	}
+
 	tracker_dbus_reply_with_query_result (rec, res);
 
-	tracker_db_free_result (res);
+	//tracker_db_free_result (res);
 }
 
 
@@ -216,10 +375,11 @@ tracker_dbus_method_search_get_snippet (DBusRec *rec)
 	DBConnection *db_con;
 	DBusError    dbus_error;
 	DBusMessage  *reply;
-	char	     *service, *uri, *str;
-	char	     *snippet, *service_id;
+	gchar	     *service, *uri, *str;
+	gchar	     *snippet, *service_id;
 
-	g_return_if_fail (rec && rec->user_data);
+	g_return_if_fail (rec);
+	g_return_if_fail (rec->user_data);
 
 	db_con = rec->user_data;
 
@@ -245,7 +405,7 @@ tracker_dbus_method_search_get_snippet (DBusRec *rec)
 		return;
 	}
 
-	if (!service)  {
+	if (!service) {
 		tracker_set_error (rec, "No service was specified");
 		return;
 	}
@@ -255,14 +415,13 @@ tracker_dbus_method_search_get_snippet (DBusRec *rec)
 		return;
 	}
 
-	if (!str || strlen (str) == 0) {
+	if (tracker_is_empty_string (str)) {
 		tracker_set_error (rec, "No search term was specified");
 		return;
 	}
 
-
 	//tracker_log ("Getting snippet with params %s, %s, %s", service, uri, str);
-
+	db_con = tracker_db_get_service_connection (db_con, service);
 	service_id = tracker_db_get_id (db_con, service, uri);
 
 	if (!service_id) {
@@ -271,19 +430,19 @@ tracker_dbus_method_search_get_snippet (DBusRec *rec)
 		return;		
 	}
 
-	char ***res;
-	const char *txt;
+	gchar ***res;
+	const gchar *txt;
 
 	snippet = NULL;
 
-	res = tracker_exec_proc (db_con->user_data2, "GetFileContents", 1, service_id);
+	res = tracker_exec_proc (db_con->blob, "GetAllContents", 1, service_id);
 	g_free (service_id);
 	
 	if (res) {
 		if (res[0][0]) {
 			txt = res[0][0];
 
-			char **array = 	tracker_parse_text_into_array (str);
+			gchar **array = 	tracker_parse_text_into_array (str);
 
 			if (array && array[0]) {
 				snippet = tracker_get_snippet (txt, array, 120);
@@ -298,10 +457,12 @@ tracker_dbus_method_search_get_snippet (DBusRec *rec)
 	}
 
 	/* do not pass NULL to dbus or it will crash */
-	if (!snippet) {
+	if (!snippet || !g_utf8_validate (snippet, -1, NULL) ) {
 		snippet = g_strdup (" ");
 	}
 
+//	tracker_debug ("snippet is %s", snippet);
+
 	reply = dbus_message_new_method_return (rec->message);
 
 	dbus_message_append_args (reply,
@@ -315,7 +476,6 @@ tracker_dbus_method_search_get_snippet (DBusRec *rec)
 }
 
 
-
 void
 tracker_dbus_method_search_files_by_text (DBusRec *rec)
 {
@@ -324,12 +484,13 @@ tracker_dbus_method_search_files_by_text (DBusRec *rec)
 	DBusMessage 	*reply;
 	DBusMessageIter iter;
 	DBusMessageIter iter_dict;
-	char 		*text;
-	int		limit, query_id, offset;
+	gchar 		*text;
+	gint		limit, query_id, offset;
 	gboolean	sort;
-	char		***res;
+	gchar		***res;
 
-	g_return_if_fail (rec && rec->user_data);
+	g_return_if_fail (rec);
+	g_return_if_fail (rec->user_data);
 
 	db_con = rec->user_data;
 
@@ -398,12 +559,13 @@ tracker_dbus_method_search_metadata (DBusRec *rec)
 	DBConnection *db_con;
 	DBusError    dbus_error;
 	DBusMessage  *reply;
-	char 	     *service, *field, *text;
-	char 	     **array;
-	int 	     limit, row_count = 0, offset;
-	char	     ***res;
+	gchar 	     *service, *field, *text;
+	gchar 	     **array;
+	gint 	     limit, row_count = 0, offset;
+	gchar	     ***res;
 
-	g_return_if_fail (rec && rec->user_data);
+	g_return_if_fail (rec);
+	g_return_if_fail (rec->user_data);
 
 	db_con = rec->user_data;
 
@@ -414,7 +576,7 @@ tracker_dbus_method_search_metadata (DBusRec *rec)
 		<method name="Metadata">
 			<arg type="s" name="service" direction="in" />
 			<arg type="s" name="field" direction="in" />
-			<arg type="s" name="search_text"  direction="in" />
+			<arg type="s" name="search_text" direction="in" />
 			<arg type="i" name="max_hits" direction="in" />
 			<arg type="as" name="result" direction="out" />
 		</method>
@@ -466,10 +628,11 @@ tracker_dbus_method_search_matching_fields (DBusRec *rec)
 {
 	DBConnection *db_con;
 	DBusError    dbus_error;
-	char 	     *text, *service, *id;
-	char	     ***res;
+	gchar 	     *text, *service, *id;
+	gchar	     ***res;
 
-	g_return_if_fail (rec && rec->user_data);
+	g_return_if_fail (rec);
+	g_return_if_fail (rec->user_data);
 
 	db_con = rec->user_data;
 
@@ -482,7 +645,7 @@ tracker_dbus_method_search_matching_fields (DBusRec *rec)
 		<method name="MatchingFields">
 			<arg type="s" name="service" direction="in" />
 			<arg type="s" name="id" direction="in" />
-			<arg type="s" name="search_text"  direction="in" />
+			<arg type="s" name="search_text" direction="in" />
 			<arg type="a{sv}" name="result" direction="out" />
 		</method>
 
@@ -504,11 +667,11 @@ tracker_dbus_method_search_matching_fields (DBusRec *rec)
 		return;
 	}
 
-	if (strlen (id) == 0) {
+	if (tracker_is_empty_string (id)) {
 		tracker_set_error (rec, "Id field must have a value");
 		return;
 	}
-
+	db_con = tracker_db_get_service_connection (db_con, service);
 	res = tracker_db_search_matching_metadata (db_con, service, id, text);
 
 	if (res) {
@@ -543,13 +706,14 @@ tracker_dbus_method_search_query (DBusRec *rec)
 {
 	DBConnection	*db_con;
 	DBusError    dbus_error;
-	char		**fields;
-	int		limit, row_count, query_id, offset;
-	char		***res;
-	char		*query, *search_text, *service, *keyword;
+	gchar		**fields;
+	gint		limit, row_count, query_id, offset;
+	gchar		***res;
+	gchar		*query, *search_text, *service, *keyword;
 	gboolean	sort_results;
 
-	g_return_if_fail (rec && rec->user_data);
+	g_return_if_fail (rec);
+	g_return_if_fail (rec->user_data);
 
 	db_con = rec->user_data;
 
@@ -602,14 +766,14 @@ tracker_dbus_method_search_query (DBusRec *rec)
 		return;
 	}
 
-	if (limit < 0) {
+	if (limit < 1) {
 		limit = 1024;
 	}
 
 	res = NULL;
 
 	if (query) {
-		char	 *str;
+		gchar	 *str;
 		GError	 *error = NULL;
 
 		tracker_log ("executing rdf query %s\n with search term %s and keyword %s", query, search_text, keyword);
@@ -642,8 +806,8 @@ tracker_dbus_method_search_query (DBusRec *rec)
 		}*/
 
 		tracker_log ("translated rdf query is \n%s\n", str);
-
-		if (search_text && (strlen (search_text) > 0)) {
+		db_con = tracker_db_get_service_connection (db_con, service);
+		if (!tracker_is_empty_string (search_text)) {
 			tracker_db_search_text (db_con, service, search_text, 0, 999999, TRUE, FALSE);
 		}
 
@@ -659,3 +823,199 @@ tracker_dbus_method_search_query (DBusRec *rec)
 
 	tracker_db_free_result (res);
 }
+
+
+/* int levenshtein ()
+ * Original license: GNU Lesser Public License
+ * from the Dixit project, (http://dixit.sourceforge.net/)
+ * Author: Octavian Procopiuc <oprocopiuc@gmail.com>
+ * Created: July 25, 2004
+ * Copied into tracker, by Edward Duffy
+ */
+
+static int
+levenshtein(char *source, char *target, int maxdist)
+{
+	char n, m;
+	int l;
+	l = strlen (source);
+	if (l > 50)
+		return -1;
+	n = l;
+
+	l = strlen (target);
+	if (l > 50)
+		return -1;
+	m = l;
+
+	if (maxdist == 0)
+		maxdist = MAX(m, n);
+	if (n == 0)
+		return MIN(m, maxdist);
+	if (m == 0)
+		return MIN(n, maxdist);
+
+	// Store the min. value on each column, so that, if it reaches
+	// maxdist, we break early.
+	char mincolval;
+
+	char matrix[51][51];
+
+	char j;
+	char i;
+	char cell;
+
+	for (j = 0; j <= m; j++)
+		matrix[0][(int)j] = j;
+
+	for (i = 1; i <= n; i++) {
+
+		mincolval = MAX(m, i);
+		matrix[(int)i][0] = i;
+
+		char s_i = source[i-1];
+
+		for (j = 1; j <= m; j++) {
+
+			char t_j = target[j-1];
+
+			char cost = (s_i == t_j ? 0 : 1);
+
+			char above = matrix[i-1][(int)j];
+			char left = matrix[(int)i][j-1];
+			char diag = matrix[i-1][j-1];
+			cell = MIN(above + 1, MIN(left + 1, diag + cost));
+
+			// Cover transposition, in addition to deletion,
+			// insertion and substitution. This step is taken from:
+			// Berghel, Hal ; Roach, David : "An Extension of Ukkonen's 
+			// Enhanced Dynamic Programming ASM Algorithm"
+			// (http://www.acm.org/~hlb/publications/asm/asm.html)
+
+			if (i > 2 && j > 2) {
+				char trans = matrix[i-2][j-2] + 1;
+				if (source[i-2] != t_j)
+					trans++;
+				if (s_i != target[j-2])
+					trans++;
+				if (cell > trans)
+					cell = trans;
+			}
+
+			mincolval = MIN(mincolval, cell);
+			matrix[(int)i][(int)j] = cell;
+		}
+
+		if (mincolval >= maxdist)
+			break;
+
+	}
+
+	if (i == n + 1)
+		return (int) matrix[(int)n][(int)m];
+	else
+		return maxdist;
+}
+
+
+void
+tracker_dbus_method_search_suggest (DBusRec *rec)
+{
+	DBusError	dbus_error;
+	DBusMessage 	*reply;
+	gchar		*term, *str;
+	gint		maxdist;
+	gint		dist, tsiz;
+	gchar		*winner_str;
+	gint		winner_dist;
+	char		*tmp;
+	int		hits;
+	GTimeVal	start, current;
+
+	/*
+		<method name="Suggest">
+			<arg type="s" name="search_text" direction="in" />
+			<arg type="i" name="maxdist" direction="in" />
+			<arg type="s" name="result" direction="out" />
+		</method>
+	*/
+
+	dbus_error_init (&dbus_error);
+	if (!dbus_message_get_args (rec->message, NULL,
+			       DBUS_TYPE_STRING, &term,
+			       DBUS_TYPE_INT32, &maxdist,
+			       DBUS_TYPE_INVALID)) {
+		tracker_set_error (rec, "DBusError: %s;%s", dbus_error.name, dbus_error.message);
+		dbus_error_free (&dbus_error);
+		return;
+	}
+
+	winner_str = NULL;
+        winner_dist = -1;  /* to initialize winner_dist with something */
+
+
+	Indexer *index = tracker->file_index;
+
+	dpiterinit (index->word_index);
+
+	g_get_current_time (&start);
+
+	str = dpiternext (index->word_index, NULL);
+	while (str != NULL) {
+		dist = levenshtein (term, str, 0);
+		if (dist != -1 && dist < maxdist) {
+			hits = 0;
+			if ((tmp = dpget (index->word_index, str, -1, 0, -1, &tsiz)) != NULL) {
+				hits = tsiz / sizeof (WordDetails);
+				free (tmp);
+				if (tsiz % sizeof (WordDetails) != 0) {
+					tracker_set_error (rec, "Possible data error from dpget  Aborting tracker_dbus_method_search_suggest.");
+					g_free (str);
+					if (winner_str) {
+						g_free (winner_str);
+					}
+					return;
+				}
+			}
+			if (hits > 0) {
+				if (winner_str == NULL) {
+					winner_str = strdup (str);
+					winner_dist = dist;
+				}
+				else if (dist < winner_dist) {
+					free (winner_str);
+					winner_str = strdup (str);
+					winner_dist = dist;
+				}
+			}
+			else {
+				tracker_log ("No hits for %s!", str);
+			}
+		}
+		free (str);
+		g_get_current_time (&current);
+		if (current.tv_sec - start.tv_sec >= 2) { /* 2 second time out */
+			tracker_log ("Timeout in tracker_dbus_method_search_suggest");
+			break;
+		}
+		str = dpiternext (index->word_index, NULL);
+	}
+
+	if (winner_str == NULL) {
+		winner_str = strdup (term);
+	}
+
+	tracker_log ("Suggested spelling for %s is %s.", term, winner_str);
+
+	reply = dbus_message_new_method_return (rec->message);
+
+	dbus_message_append_args (reply,
+	  			  DBUS_TYPE_STRING, &winner_str,
+	  			  DBUS_TYPE_INVALID);
+	free (winner_str);
+
+	dbus_connection_send (rec->connection, reply, NULL);
+
+	dbus_message_unref (reply);
+
+}
