@@ -30,6 +30,9 @@
 #endif
 
 #include <assert.h>
+#ifdef OS_WIN32
+#include "../trackerd/mingw-compat.h"
+#endif
 #include "xdgmimemagic.h"
 #include "xdgmimeint.h"
 #include <stdio.h>
@@ -47,7 +50,9 @@
 #define	TRUE	(!FALSE)
 #endif
 
-extern int errno;
+#if !defined getc_unlocked && !defined HAVE_GETC_UNLOCKED
+# define getc_unlocked(fp) getc (fp)
+#endif
 
 typedef struct XdgMimeMagicMatch XdgMimeMagicMatch;
 typedef struct XdgMimeMagicMatchlet XdgMimeMagicMatchlet;
@@ -315,7 +320,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
   int c;
   int end_of_file;
   int indent = 0;
-  unsigned int bytes_read;
+  int bytes_read;
 
   assert (magic_file != NULL);
 
@@ -454,21 +459,17 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 
   if (c == '+')
     {
-      int range_length;
-
-      range_length = _xdg_mime_magic_read_a_number (magic_file, &end_of_file);
+      matchlet->range_length = _xdg_mime_magic_read_a_number (magic_file, &end_of_file);
       if (end_of_file)
 	{
 	  _xdg_mime_magic_matchlet_free (matchlet);
 	  return XDG_MIME_MAGIC_EOF;
 	}
-      if (range_length == -1)
+      if (matchlet->range_length == -1)
 	{
 	  _xdg_mime_magic_matchlet_free (matchlet);
 	  return XDG_MIME_MAGIC_ERROR;
 	}
-      /* now we are sure that range_length is positive */
-      matchlet->range_length = (unsigned int) range_length;
       c = getc_unlocked (magic_file);
     }
 
@@ -478,9 +479,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
       /* We clean up the matchlet, byte swapping if needed */
       if (matchlet->word_size > 1)
 	{
-#if LITTLE_ENDIAN
-	  unsigned int i;
-#endif
+	  int i;
 	  if (matchlet->value_length % matchlet->word_size != 0)
 	    {
 	      _xdg_mime_magic_matchlet_free (matchlet);
@@ -525,7 +524,7 @@ _xdg_mime_magic_matchlet_compare_to_data (XdgMimeMagicMatchlet *matchlet,
 					  const void           *data,
 					  size_t                len)
 {
-  unsigned int i, j;
+  int i, j;
   for (i = matchlet->offset; i < matchlet->offset + matchlet->range_length; i++)
     {
       int valid_matchlet = TRUE;
@@ -666,28 +665,15 @@ _xdg_mime_magic_lookup_data (XdgMimeMagic *mime_magic,
   XdgMimeMagicMatch *match;
   const char *mime_type;
   int n;
-  int priority;
-  int had_match;
 
   mime_type = NULL;
-  priority = 0;
-  had_match = 0;
   for (match = mime_magic->match_list; match; match = match->next)
     {
       if (_xdg_mime_magic_match_compare_to_data (match, data, len))
 	{
-	  if (!had_match || match->priority > priority ||
-	      (mime_type != NULL && _xdg_mime_mime_type_subclass (match->mime_type, mime_type)))
-	    {
-	      mime_type = match->mime_type;
-	      priority = match->priority;
-	    }
-	  else if (had_match && match->priority == priority)
-	    /* multiple unrelated patterns with the same priority matched,
-	     * so we can't tell what type this is. */
-	    mime_type = NULL;
-
-	  had_match = 1;
+	  if ((mime_type == NULL) || (_xdg_mime_mime_type_subclass (match->mime_type, mime_type))) {
+	    mime_type = match->mime_type;
+	  }
 	}
       else 
 	{
